"""
################################################################################
# Copyright (c) 2003, Pfizer
# Copyright (c) 2001, Cayce Ullman.
# Copyright (c) 2001, Brian Matthews.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
#
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# Neither the name of actzero, inc. nor the names of its contributors may
# be used to endorse or promote products derived from this software without
# specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
################################################################################
"""

ident = '$Id: Config.py,v 1.9 2004/01/31 04:20:05 warnes Exp $'
from version import __version__

import copy, socket
from types import *

from NS import NS 

################################################################################
# Configuration class
################################################################################

class SOAPConfig:
    __readonly = ('SSLserver', 'SSLclient', 'GSIserver', 'GSIclient')

    def __init__(self, config = None, **kw):
        d = self.__dict__

        if config:
            if not isinstance(config, SOAPConfig):
                raise AttributeError, \
                    "initializer must be SOAPConfig instance"

            s = config.__dict__

            for k, v in s.items():
                if k[0] != '_':
                    d[k] = v
        else:
            # Setting debug also sets returnFaultInfo, 
            # dumpHeadersIn, dumpHeadersOut, dumpSOAPIn, and dumpSOAPOut
            self.debug = 0
            self.dumpFaultInfo = 1
            # Setting namespaceStyle sets typesNamespace, typesNamespaceURI,
            # schemaNamespace, and schemaNamespaceURI
            self.namespaceStyle = '1999'
            self.strictNamespaces = 0
            self.typed = 1
            self.buildWithNamespacePrefix = 1
            self.returnAllAttrs = 0

            # Strict checking of range for floats and doubles
            self.strict_range = 0

            # Default encoding for dictionary keys
            self.dict_encoding = 'ascii'

            # New argument name handling mechanism.  See
            # README.MethodParameterNaming for details
            self.specialArgs = 1

            # If unwrap_results=1 and there is only element in the struct,
            # SOAPProxy will assume that this element is the result
            # and return it rather than the struct containing it.
            # Otherwise SOAPproxy will return the struct with all the
            # elements as attributes.
            self.unwrap_results = 1

            # Automatically convert SOAP complex types, and
            # (recursively) public contents into the corresponding
            # python types. (Private subobjects have names that start
            # with '_'.)
            #
            # Conversions:
            # - faultType    --> raise python exception
            # - arrayType    --> array
            # - compoundType --> dictionary
            #
            self.simplify_objects = 0

            # Per-class authorization method.  If this is set, before
            # calling a any class method, the specified authorization
            # method will be called.  If it returns 1, the method call
            # will proceed, otherwise the call will throw with an
            # authorization error.
            self.authMethod = None

            # Globus Support if pyGlobus.io available
            try:
                from pyGlobus import io;
                d['GSIserver'] = 1
                d['GSIclient'] = 1
            except:
                d['GSIserver'] = 0
                d['GSIclient'] = 0
                

            # Server SSL support if M2Crypto.SSL available
                d['SSLserver'] = 0

            # Client SSL support if socket.ssl available
            try:
                from socket import ssl
                d['SSLclient'] = 1
            except:
                d['SSLclient'] = 0

            # Fields pertaining to compression of HTTP messages. If
            # accept_compressed is set to True, outgoing messages will ask the
            # server to send back gzip compressed messages. If send_compressed
            # is set to True, outgoing messages are themselves gzip compressed.
            self.accept_compressed = False
            self.send_compressed = False

        for k, v in kw.items():
            if k[0] != '_':
                setattr(self, k, v)

    def __setattr__(self, name, value):
        if name in self.__readonly:
            raise AttributeError, "readonly configuration setting"

        d = self.__dict__

        if name in ('typesNamespace', 'typesNamespaceURI',
                    'schemaNamespace', 'schemaNamespaceURI'):

            if name[-3:] == 'URI':
                base, uri = name[:-3], 1
            else:
                base, uri = name, 0

            if type(value) == StringType:
                if NS.NSMAP.has_key(value):
                    n = (value, NS.NSMAP[value])
                elif NS.NSMAP_R.has_key(value):
                    n = (NS.NSMAP_R[value], value)
                else:
                    raise AttributeError, "unknown namespace"
            elif type(value) in (ListType, TupleType):
                if uri:
                    n = (value[1], value[0])
                else:
                    n = (value[0], value[1])
            else:
                raise AttributeError, "unknown namespace type"

            d[base], d[base + 'URI'] = n

            try:
                d['namespaceStyle'] = \
                    NS.STMAP_R[(d['typesNamespace'], d['schemaNamespace'])]
            except:
                d['namespaceStyle'] = ''

        elif name == 'namespaceStyle':
            value = str(value)

            if not NS.STMAP.has_key(value):
                raise AttributeError, "unknown namespace style"

            d[name] = value
            n = d['typesNamespace'] = NS.STMAP[value][0]
            d['typesNamespaceURI'] = NS.NSMAP[n]
            n = d['schemaNamespace'] = NS.STMAP[value][1]
            d['schemaNamespaceURI'] = NS.NSMAP[n]

        elif name == 'debug':
            d[name]                     = \
                d['returnFaultInfo']    = \
                d['dumpHeadersIn']      = \
                d['dumpHeadersOut']     = \
                d['dumpSOAPIn']         = \
                d['dumpSOAPOut']        = value
            
        else:
            d[name] = value


Config = SOAPConfig()



#!/usr/bin/python
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utility functions for interacting with SOAPpy type objects."""

__author__ = 'api.jdilallo@gmail.com (Joseph DiLallo)'


def GetArrayItemTypeName(type_name, ns, soappy_service):
  """Returns the name of the SOAP type which the items in an array represent.

  Python arrays can represent two distinct kinds of SOAP objects: SOAP encoded
  arrays or complex type elements with a maxOccurs field greater than 1. In the
  first case, the type_name given represents the SOAP encoded array type and the
  type returned will be the content's type. In the second case, or any case
  where the type_name given is not a SOAP encoded array, the type_name given is
  the one that will be returned.

  Args:
    type_name: string The name of the WSDL-defined type of the array.
    ns: string The namespace this WSDL-defined type belongs to.
    soappy_service: SOAPpy.WSDL.proxy The SOAPpy service object which contains
                    the WSDL definitions.

  Returns:
    string The type name of the array's contents.
  """
  try:
    array_type = GetTypeFromSoappyService(type_name, ns, soappy_service)
    attr_contents = array_type.content.derivation.attr_content
    if attr_contents:
      raw_type_name = attr_contents[0].attributes[
          'http://schemas.xmlsoap.org/wsdl/']['arrayType']
      return raw_type_name[raw_type_name.find(':') + 1:-2]
    else:
      return type_name
  except KeyError:
    return type_name
  except AttributeError:
    return type_name


def IsASuperType(soappy_service, sub_type, ns, super_type):
  """Determines if one type is a supertype of another type.

  Any case where the sub_type cannot be traced through to super_type is
  considered to be false.

  Args:
    soappy_service: SOAPpy.WSDL.proxy The SOAPpy service object which contains
                    the WSDL definitions.
    sub_type: A string representing a type that may be extending super_type.
    ns: A string representing the namespace sub_type belongs to.
    super_type: A string representing a type that may be extended by sub_type.

  Returns:
    bool Whether super_type is really a supertype of sub_type.
  """
  if sub_type == super_type: return True

  try:
    if IsASubType(sub_type, ns, soappy_service):
      complex_type = GetTypeFromSoappyService(sub_type, ns, soappy_service)
      return IsASuperType(
          soappy_service,
          complex_type.content.derivation.attributes['base'].getName(), ns,
          super_type)
    else:
      return False
  except KeyError:
    # The given sub_type does not exist in the WSDL definitions.
    return False


def IsASubType(type_name, ns, soappy_service):
  """Determines if a WSDL-defined type is extending another WSDL-defined type.

  Args:
    type_name: string The name of the WSDL-defined type.
    ns: string The namespace this WSDL-defined type belongs to.
    soappy_service: SOAPpy.WSDL.proxy The SOAPpy service object which contains
                    the WSDL definitions.

  Returns:
    boolean Whether the given type is extending another type.
  """
  return hasattr(GetTypeFromSoappyService(
      type_name, ns, soappy_service).content, 'derivation')


def IsAnArrayType(type_name, ns, soappy_service):
  """Determines if a complex type represents a SOAP-encoded array.

  Args:
    type_name: string The name of the WSDL-defined type.
    ns: string The namespace this WSDL-defined type belongs to.
    soappy_service: SOAPpy.WSDL.proxy The SOAPpy service object which contains
                    the WSDL definitions.

  Returns:
    boolean Whether the given type represents an array.
  """
  if type_name == 'Array':
    return True
  try:
    wsdl_type_def = GetTypeFromSoappyService(type_name, ns, soappy_service)
    if hasattr(wsdl_type_def.content, 'derivation'):
      # This is an extension of another type.
      return IsAnArrayType(
          wsdl_type_def.content.derivation.attributes['base'].getName(),
          wsdl_type_def.content.derivation.attributes[
              'base'].getTargetNamespace(), soappy_service)
    else:
      return False
  except KeyError:
    return False


def GetTypeFromSoappyService(type_name, ns, soappy_service):
  """Digs in a SOAPpy service proxy and returns the object representing a type.

  Args:
    type_name: string The name of the WSDL-defined type to search for.
    ns: string The namespace the given WSDL-defined type belongs to.
    soappy_service: SOAPpy.WSDL.Proxy The SOAPpy service object encapsulating
                    the information stored in the WSDL.

  Returns:
    mixed The object created by SOAPpy representing the given type. May be
    either a SOAPpy.wstools.XMLSchema.SimpleType or
    SOAPpy.wstools.XMLSchema.ComplexType object.
  """
  return soappy_service.wsdl.types[ns].types[type_name]


def GenKeyOrderAttrs(soappy_service, ns, type_name):
  """Generates the order and attributes of keys in a complex type.

  Args:
    soappy_service: SOAPpy.WSDL.Proxy The SOAPpy service object encapsulating
                    the information stored in the WSDL.
    ns: string The namespace the given WSDL-defined type belongs to.
    type_name: string The name of the WSDL-defined type to search for.

  Returns:
    list A list of dictionaries containing the attributes of keys within a
    complex type, in order.
  """
  complex_type = soappy_service.wsdl.types[ns].types[type_name]

  if IsASubType(type_name, ns, soappy_service):
    # This is an extension of another type.
    key_order = GenKeyOrderAttrs(
        soappy_service,
        complex_type.content.derivation.attributes['base'].getTargetNamespace(),
        complex_type.content.derivation.attributes['base'].getName())
    if hasattr(complex_type.content.derivation.content, 'content'):
      key_order.extend([element.attributes for element in
                        complex_type.content.derivation.content.content])
    return key_order
  else:
    # This is a base type.
    return [element.attributes for element in complex_type.content.content]


def PruneKeyOrder(key_order, soappy_struct_object):
  """Creates a new key order of only keys that the complex type used.

  Args:
    key_order: list A list of the keys these complex type contains, in order.
              These keys are not namespaced, whereas the ones in the given
              object may be.
    soappy_struct_object: SOAPpy.Types.structType The complex type packed into a
                          SOAPpy object. Already has within it all of the keys
                          it is going to use.

  Returns:
    list A new key order containing only the keys used in the given object.
    These keys may be namespaced; they appear as they are in the given object.
  """
  new_key_order = []
  key_to_namespaced_key = {}
  for namespaced_key in soappy_struct_object._data.keys():
    if ':' in namespaced_key:
      key_to_namespaced_key[namespaced_key[
          namespaced_key.find(':') + 1:]] = namespaced_key
    else:
      key_to_namespaced_key[namespaced_key] = namespaced_key
  for namespaced_attribute in dir(soappy_struct_object):
    if ':' in namespaced_attribute:
      key_to_namespaced_key[namespaced_attribute[
          namespaced_attribute.find(':') + 1:]] = namespaced_attribute
    else:
      key_to_namespaced_key[namespaced_attribute] = namespaced_attribute

  for key in key_order:
    if key in key_to_namespaced_key:
      new_key_order.append(key_to_namespaced_key[key])
  return new_key_order


def GetComplexFieldTypeByFieldName(key, type_name, ns, soappy_service):
  """Returns the type of a field within a complex type by its key name.

  Args:
    key: string The name of the field within the given complex type whose type
         is being looked up.
    type_name: string The name of the encapsulating complex type.
    ns: string The namespace the encapsulating compelx type belongs to.
    soappy_service: SOAPpy.WSDL.Proxy The SOAPpy service object containing the
                    descriptions of these types.

  Returns:
    TypeDescriptionComponent The type of object stored in the field with the
    given name.

  Raises:
    TypeError: if the given key is not within the given complex type.
  """
  for param in GenKeyOrderAttrs(soappy_service, ns, type_name):
    if param['name'] == key:
      return param['type']
  raise TypeError('There is no field with the name %s in complex type %s.'
                  % (key, type_name))


def GetComplexFieldNamespaceByFieldName(field, type_name, ns, soappy_service):
  """Returns the namespace of the type which defines a field in a hierarchy.

  Args:
    field: string The name of the field within the given complex type whose
           namespace is being looked up.
    type_name: string The name of the encapsulating complex type.
    ns: string The namespace the encapsulating compelx type belongs to.
    soappy_service: SOAPpy.WSDL.Proxy The SOAPpy service object containing the
                    descriptions of these types.

  Returns:
    string The URL of the namespace this field was defined within.

  Raises:
    TypeError: if the given field is not within the given complex type.
  """
  type_obj = soappy_service.wsdl.types[ns].types[type_name]

  if IsASubType(type_name, ns, soappy_service):
    if hasattr(type_obj.content.derivation.content, 'content'):
      for element in type_obj.content.derivation.content.content:
        if element.attributes['name'] == field: return ns
    try:
      return GetComplexFieldNamespaceByFieldName(
          field,
          type_obj.content.derivation.attributes['base'].getName(),
          type_obj.content.derivation.attributes['base'].getTargetNamespace(),
          soappy_service)
    except TypeError:
      raise TypeError('There is no field with the name %s in complex type %s.'
                      % (field, type_name))
  else:
    for element in type_obj.content.content:
      if element.attributes['name'] == field: return ns
    raise TypeError('There is no field with the name %s in complex type %s.'
                    % (field, type_name))


def GetExplicitType(obj, type_name, ns, soappy_service):
  """Returns the WSDL-defined type set within the given object, if one exists.

  Args:
    obj: dict The python representation of an object of the given type.
    type_name: string The name of the type the given object represents.
    ns: string The namespace the given type belongs to.
    soappy_service: SOAPpy.WSDL.Proxy The SOAPpy service object encapsulating
                    the WSDL definitions.

  Returns:
    tuple(string, string) The name of the type explicitly set within the given
    object, followed by the key used to specify this type. If no type is set
    within the given object, returns the tuple (None, None).
  """
  if 'xsi_type' in obj:
    return (obj['xsi_type'], 'xsi_type')
  elif 'type' in obj and not _HasNativeType(type_name, ns, soappy_service):
    return (obj['type'], 'type')
  else:
    for item in obj:
      if item.find('.Type') > -1 or item.find('_Type') > -1:
        return (obj[item], item)
  return (None, None)


def _HasNativeType(type_name, ns, soappy_service):
  """Checks a given WSDL-defined type for a field named 'type'.

  Args:
    type_name: string The name of the type to search.
    ns: string The namespace the given type belongs to.
    soappy_service: SOAPpy.WSDL.Proxy The SOAPpy service object encapsulating
                    the WSDL definitions.

  Returns:
    bool Whether or not the given type has a field named 'type'.
  """
  params = GenKeyOrderAttrs(soappy_service, ns, type_name)
  for param in params:
    if param['name'] == 'type': return True
  return False



#!/usr/bin/python
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Validation functions."""

__author__ = 'api.sgrinberg@gmail.com (Stan Grinberg)'

from adspygoogle.common import ETREE
from adspygoogle.common import PYXML
from adspygoogle.common import Utils
from adspygoogle.common.Errors import ValidationError
from adspygoogle.common.soappy import SoappyUtils


def ValidateRequiredHeaders(headers, required_headers):
  """Sanity check for required authentication elements.

  The required headers may contain several possible header combinations, only
  one of which must be satisfied to make successful API request. In order for
  any single combination to be satisfied, all of the headers it specifies must
  exist as keys within the headers dict and each entry must contain data.

  Args:
    headers: A dict containing authentication headers.
    required_headers: A tuple containing valid combinations of headers. Each
                      combination of headers is represented as a tuple of
                      strings. e.g. (('Name', 'Password'), ('Name', 'Token'))

  Raises:
    ValidationError: The given authentication headers are not sufficient to make
                     requests against this API.
  """
  is_valid = False
  for headers_set in required_headers:
    is_valid_set = True
    for key in headers_set:
      if key not in headers:
        is_valid_set = False
      elif isinstance(headers[key], basestring) and not headers[key].strip():
        is_valid_set = False
    if is_valid_set:
      is_valid = True
      break

  if not is_valid:
    msg = ('Required authentication header is missing. Valid options for '
           'headers are %s.' % str(required_headers))
    raise ValidationError(msg)


def IsConfigUserInputValid(user_input, valid_el):
  """Determines if user input within configuration scripts is valid.

  Each time a choice is presented to the user, a set of allowed values specific
  to that interaction is passed into this function.

  Args:
    user_input: The string of user input.
    valid_el: A list of valid elements for the current interaction.

  Returns:
    bool True if user input is valid, False otherwise.
  """
  if not user_input: return False

  try:
    valid_el.index(str(user_input))
  except ValueError:
    return False
  return True


def ValidateConfigXmlParser(xml_parser):
  """Checks that the XML parser set in the configuration is a valid choice.

  Args:
    xml_parser: A string specifying which XML parser to use.

  Raises:
    ValidationError: The given XML parser is not supported by this library.
  """
  if (not isinstance(xml_parser, basestring) or
      not IsConfigUserInputValid(xml_parser, [PYXML, ETREE])):
    msg = ('Invalid input for %s \'%s\', expecting %s or %s of type <str>.'
           % (type(xml_parser), xml_parser, PYXML, ETREE))
    raise ValidationError(msg)


def ValidateTypes(vars_tpl):
  """Checks that each variable in a set of variables is the correct type.

  Args:
    vars_tpl: A tuple containing a set of variables to check.

  Raises:
    ValidationError: The given object was not one of the given accepted types.
  """
  for var, var_types in vars_tpl:
    if not isinstance(var_types, tuple):
      var_types = (var_types,)
    for var_type in var_types:
      if isinstance(var, var_type):
        return
    msg = ('The \'%s\' is of type %s, expecting one of %s.'
           % (var, type(var), var_types))
    raise ValidationError(msg)


def _SoappySanityCheckComplexType(soappy_service, obj, ns, xsi_type):
  """Validates a dict representing a complex type against its WSDL definition.

  Args:
    soappy_service: SOAPpy.WSDL.Proxy The SOAPpy service object containing the
                    descriptions of all WSDL-defined types.
    obj: A dict that should represent an instance of the given type.
    ns: string The namespace the given type belongs to.
    xsi_type: A string specifying the name of a complex type defined in the
              WSDL.

  Raises:
    ValidationError: The given object is not an acceptable representation of the
                     given WSDL-defined complex type.
  """
  ValidateTypes(((obj, dict),))
  obj_contained_type, type_key = SoappyUtils.GetExplicitType(obj, xsi_type, ns,
                                                             soappy_service)

  if obj_contained_type and not obj_contained_type == xsi_type:
    try:
      SoappyUtils.GetTypeFromSoappyService(obj_contained_type, ns,
                                           soappy_service)
    except KeyError:
      raise ValidationError('Object of class \'%s\' has an explicit type of '
                            '\'%s\', but this explicit type is not defined in '
                            'the WSDL.' % (xsi_type, obj_contained_type))
    if not SoappyUtils.IsASuperType(soappy_service, obj_contained_type, ns,
                                    xsi_type):
      raise ValidationError('Expecting type of \'%s\' but given type of class '
                            '\'%s\'.' % (xsi_type, obj_contained_type))
    xsi_type = obj_contained_type

  parameters = SoappyUtils.GenKeyOrderAttrs(soappy_service, ns, xsi_type)
  for key in obj:
    if obj[key] is None or key == type_key:
      continue
    found = False
    for parameter, param_type, max_occurs in [
        (param['name'], param['type'], param['maxOccurs']) for
        param in parameters]:
      if parameter == key:
        found = True
        if not max_occurs.isdigit() or int(max_occurs) > 1:
          # This parameter should be a list.
          if isinstance(obj[key], (list, tuple)):
            for item in obj[key]:
              SoappySanityCheck(soappy_service, item,
                                param_type.getTargetNamespace(),
                                param_type.getName())
          else:
            raise ValidationError('Field \'%s\' in complex type \'%s\' should '
                                  'be a list but value \'%s\' is a \'%s\' '
                                  'instead.'
                                  % (key, xsi_type, obj[key], type(obj[key])))
        else:
          SoappySanityCheck(soappy_service, obj[key],
                            param_type.getTargetNamespace(),
                            param_type.getName())
        break
    if not found:
      raise ValidationError('Field \'%s\' is not in type \'%s\'.'
                            % (key, xsi_type))


def _SoappySanityCheckSimpleType(obj, xsi_type):
  """Validates a string representing a simple type against its WSDL definition.

  Args:
    obj: String representing the given simple type.
    xsi_type: A string specifying the simple type name defined in the WSDL.

  Raises:
    ValidationError: The given object is not an acceptable representation of the
                     given WSDL-defined simple type.
  """
  try:
    ValidateTypes(((obj, (str, unicode)),))
  except ValidationError:
    raise ValidationError('Simple type \'%s\' should be a string but value '
                          '\'%s\' is a \'%s\' instead.' %
                          (xsi_type, obj, type(obj)))


def _SoappySanityCheckArray(soappy_service, obj, ns, type_name):
  """Validates a list representing an array type against its WSDL definition.

  Args:
    soappy_service: SOAPpy.WSDL.Proxy The SOAPpy service object containing the
                    descriptions of all WSDL-defined types.
    obj: Object to be validated. Depending on the WSDL-defined type this object
         represents, the data type will vary. It should always be either a
         dictionary, list, or string no matter what WSDL-defined type it is.
    ns: string The namespace the given type belongs to.
    type_name: A string specifying the type name defined in the WSDL.

  Raises:
    ValidationError if the given object is not a valid representation of the
    given list type.
  """
  try:
    ValidateTypes(((obj, list),))
  except ValidationError:
    raise ValidationError('Type \'%s\' should be a list but value '
                          '\'%s\' is a \'%s\' instead.' %
                          (type_name, obj, type(obj)))
  if Utils.IsBaseSoapType(type_name):
    for item in obj:
      if item is None: continue
      try:
        ValidateTypes(((item, (str, unicode)),))
      except ValidationError:
        raise ValidationError('The items in array \'%s\' must all be strings. '
                              'Value \'%s\' is of type \'%s\'.'
                              % (type_name, item, type(item)))
  else:
    for item in obj:
      if item is None: continue
      SoappySanityCheck(soappy_service, item, ns,
                        SoappyUtils.GetArrayItemTypeName(type_name, ns,
                                                         soappy_service))


def SoappySanityCheck(soappy_service, obj, ns, obj_type, max_occurs='1'):
  """Validates any given object against its WSDL definition.

  This method considers None and the empty string to be a valid representation
  of any type.

  Args:
    soappy_service: SOAPpy.WSDL.Proxy An object encapsulating a SOAP service.
    obj: Object to be validated. Depending on the WSDL-defined type this object
         represents, the data type will vary. It should always be either a
         dictionary, list, or string no matter what WSDL-defined type it is.
    ns: string The namespace the given type belongs to.
    obj_type: A string specifying the type name defined in the WSDL.
    max_occurs: string The maxOccurs attribute for this object.

  Raises:
    ValidationError: The given type has no definition in the WSDL or the given
                     object is not a valid representation of the type defined in
                     the WSDL.
  """
  if obj in (None, ''):
    return
  elif Utils.IsBaseSoapType(obj_type):
    try:
      ValidateTypes(((obj, (str, unicode)),))
    except ValidationError:
      raise ValidationError('Objects of type \'%s\' should be a string but '
                            'value \'%s\' is a \'%s\' instead.' %
                            (obj_type, obj, type(obj)))
  else:
    try:
      soap_type = soappy_service.wsdl.types[ns].types[obj_type].tag
      if soap_type == 'simpleType':
        _SoappySanityCheckSimpleType(obj, obj_type)
      elif soap_type == 'complexType':
        if (SoappyUtils.IsAnArrayType(obj_type, ns, soappy_service) or
            not max_occurs.isdigit() or int(max_occurs) > 1):
          _SoappySanityCheckArray(soappy_service, obj, ns, obj_type)
        else:
          _SoappySanityCheckComplexType(soappy_service, obj, ns, obj_type)
      else:
        raise ValidationError('Unrecognized type definition tag in WSDL: \'%s\''
                              % soap_type)
    except KeyError:
      raise ValidationError('This type is not defined in the WSDL: \'%s\''
                            % obj_type)



#!/usr/bin/python
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Settings and configurations for the client library."""

__author__ = ('Stan Grinberg',
              'Vincent Tsao',
              'Nicholas Chen')

import os
import pickle
import sys

from adspygoogle.common import GenerateLibSig
from adspygoogle.common import Utils
from adspygoogle.common import VERSION
from adspygoogle.common.Errors import MissingPackageError


LIB_HOME = os.path.abspath(os.path.join(os.path.dirname(__file__)))
LIB_NAME = 'DFP API Python Client Library'
LIB_SHORT_NAME = 'DfpApi-Python'
LIB_URL = 'http://code.google.com/p/google-api-ads-python'
LIB_AUTHOR = 'Nicholas Chen'
LIB_VERSION = '9.10.0'
LIB_MIN_COMMON_VERSION = '3.1.1'
LIB_SIG = GenerateLibSig(LIB_SHORT_NAME, LIB_VERSION)

if VERSION < LIB_MIN_COMMON_VERSION:
  msg = ('Unsupported version of the core module is detected. Please download '
         'the latest version of client library at %s.' % LIB_URL)
  raise MissingPackageError(msg)

# Tuple of strings representing API versions.
API_VERSIONS = ('v201208', 'v201211', 'v201302', 'v201306', 'v201308')
DEFAULT_API_VERSION = API_VERSIONS[-1]

# Accepted combinations of headers which user has to provide. Either one of
# these is required in order to make a succesful API request.
REQUIRED_SOAP_HEADERS = (('email', 'password', 'applicationName'),
                         ('authToken', 'applicationName'),
                         ('oauth2credentials', 'applicationName'))

AUTH_TOKEN_SERVICE = 'gam'
AUTH_TOKEN_EXPIRE = 60 * 60 * 23

ERROR_TYPES = []
for item in Utils.GetDataFromCsvFile(os.path.join(LIB_HOME, 'data',
                                                  'error_types.csv')):
  ERROR_TYPES.append(item[0])



#!/usr/bin/python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Setup script for the Google Ads Python Client Library."""

__author__ = 'api.msaniscalchi@gmail.com (Mark Saniscalchi)'

import os
import sys

import warnings

warnings.filterwarnings(message='Can\'t import.*', action='ignore')

from adspygoogle import LIB_PYPI_AUTHOR
from adspygoogle import LIB_PYPI_AUTHOR_EMAIL
from adspygoogle import LIB_PYPI_NAME
from adspygoogle import LIB_PYPI_URL
from adspygoogle import LIB_PYPI_VERSION

from setuptools import setup

PACKAGES = ['adspygoogle', 'adspygoogle.common', 'adspygoogle.common.https',
            'adspygoogle.common.soappy', 'adspygoogle.adwords',
            'adspygoogle.adwords.util', 'adspygoogle.dfa', 'adspygoogle.dfp',
            'adspygoogle.SOAPpy', 'adspygoogle.SOAPpy.wstools',
            'adspygoogle.scripts.adspygoogle.adwords',
            'adspygoogle.scripts.adspygoogle.common',
            'adspygoogle.scripts.adspygoogle.dfa',
            'adspygoogle.scripts.adspygoogle.dfp']

PACKAGE_DATA = {'adspygoogle.adwords': [os.path.join('data', '*')],
                'adspygoogle.dfa': [os.path.join('data', '*')],
                'adspygoogle.dfp': [os.path.join('data', '*')]}

DEPENDENCIES = ['PyXML', 'ElementTree', 'cElementTree', 'Epydoc', 'fpconst',
                'mock', 'google-api-python-client', 'pytz']

# For manual installation, optionally remove PyXML and cElementTree for
# installation of adspygoogle on incompatible operating systems.
if '--no_PyXML' in sys.argv:
  print 'Not installing PyXML.'
  sys.argv.remove('--no_PyXML')
  DEPENDENCIES.remove('PyXML')

if '--no_cElementTree' in sys.argv:
  print 'Not installing cElementTree.'
  sys.argv.remove('--no_cElementTree')
  DEPENDENCIES.remove('cElementTree')

setup(name='adspygoogle',
      version=LIB_PYPI_VERSION,
      description=LIB_PYPI_NAME,
      author=LIB_PYPI_AUTHOR,
      author_email=LIB_PYPI_AUTHOR_EMAIL,
      maintainer=LIB_PYPI_AUTHOR,
      maintainer_email=LIB_PYPI_AUTHOR_EMAIL,
      url=LIB_PYPI_URL,
      license='Apache License 2.0',
      long_description=open('README.txt').read(),
      packages=PACKAGES,
      package_data=PACKAGE_DATA,
      platforms='any',
      keywords='adwords adxbuyer dfp dfa google',
      install_requires=DEPENDENCIES)



#!/usr/bin/python
#
# Copyright 2010 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Classes for handling errors."""

__author__ = 'api.sgrinberg@gmail.com (Stan Grinberg)'

from adspygoogle.common.Errors import DetailError
from adspygoogle.common.Errors import Error
from adspygoogle.dfp import ERROR_TYPES


class DfpError(Error):

  """Implements DfpError.

  Responsible for handling error.
  """

  def __init(self, msg):
    super(DfpError, self).__init__()

  def __str__(self):
    return super(DfpError, self).__str__()

  def __call__(self):
    return super(DfpError, self).__call__()


class DfpDetailError(DetailError):

  """Implements DfpDetailError.

  Responsible for handling detailed ApiException error.
  """

  def __init__(self, **error):
    super(DfpDetailError, self).__init__()
    self.__error = error
    for key in self.__error:
      self.__dict__.__setitem__(key, self.__error[key])

  def __call__(self):
    return (self.__error,)


class DfpApiError(DfpError):

  """Implements DfpApiError.

  Responsible for handling API exception.
  """

  def __init__(self, fault):
    (self.__fault, self.fault_code, self.fault_string) = (fault, '', '')
    if 'faultcode' in fault: self.fault_code = fault['faultcode']
    if 'faultstring' in fault: self.fault_string = fault['faultstring']

    (self.message, self.trigger, self.errors) = ('', '', [])
    if 'detail' in fault:
      if 'message' in fault['detail']: self.message = fault['detail']['message']
    if not self.message: self.message = self.fault_string

    errors = [None]
    if 'detail' in fault and 'errors' in fault['detail']:
      errors = fault['detail']['errors']
    elif 'detail' not in fault:
      errors[0] = {}
    else:
      errors[0] = fault['detail']
    if isinstance(errors, list):
      for error in errors:
        # Keys need to be of type str not unicode.
        error_dct = dict([(str(key), value) for key, value in error.items()])
        self.errors.append(DfpDetailError(**error_dct))
    else:
      # Keys need to be of type str not unicode.
      error_dct = dict([(str(key), value) for key, value in errors.items()])
      self.errors.append(DfpDetailError(**error_dct))

  def __str__(self):
    return self.fault_string

  def __call__(self):
    return (self.__fault,)


class DfpRequestError(DfpApiError):

  """Implements DfpRequestError.

  Responsible for handling request error."""

  pass


class DfpGoogleInternalError(DfpApiError):

  """Implements DfpGoogleInternalError.

  Responsible for handling Google internal error.
  """

  pass


class DfpAuthenticationError(DfpApiError):

  """Implements DfpAuthenticationError.

  Responsible for handling authentication error.
  """

  pass


class DfpAccountError(DfpApiError):

  """Implements DfpAccountError.

  Responsible for handling account error.
  """

  pass


class DfpWebpageError(DfpApiError):

  """Implements DfpWebpageError.

  Responsible for handling webpage error.
  """

  pass


class DfpBillingError(DfpApiError):

  """Implements DfpBillingError.

  Responsible for handling billing error.
  """

  pass


# Map error codes and types to their corresponding classes.
ERRORS = {}
for index in ERROR_TYPES:
  if index in ('AdUnitAfcSizeError', 'AdUnitCodeError', 'AdUnitHierarchyError',
               'ApiError', 'ApiVersionError', 'CommonError', 'CreativeError',
               'CustomTargetingError', 'DayPartTargetingError', 'FileError',
               'FlashCreativeError', 'ForecastError', 'GeoTargetingError',
               'ImageError', 'InvalidUrlError', 'InventoryTargetingError',
               'InventoryUnitError', 'LineItemCreativeAssociationError',
               'LineItemCreativeAssociationOperationError',
               'LineItemFlightDateError', 'LineItemOperationError',
               'NotNullError', 'NullError', 'OrderActionError', 'OrderError',
               'ParseError', 'PermissionError',
               'PublisherQueryLanguageContextError',
               'PublisherQueryLanguageSyntaxError', 'RangeError', 'RegExError',
               'RequiredCollectionError', 'RequiredError',
               'RequiredNumberError', 'RequiredSizeError', 'ReportError',
               'ReservationDetailsError', 'StatementError', 'StringLengthError',
               'TypeError', 'UniqueError', 'UserDomainTargetingError'):
    ERRORS[index] = DfpRequestError
  elif index in ('InternalApiError', 'QuotaError', 'ServerError'):
    ERRORS[index] = DfpGoogleInternalError
  elif index in ('AuthenticationError', 'InvalidEmailError'):
    ERRORS[index] = DfpAuthenticationError



#! /usr/bin/env python
"""Logging"""
import sys


class ILogger:
    '''Logger interface, by default this class
    will be used and logging calls are no-ops.
    '''
    level = 0
    def __init__(self, msg):
        return
    def warning(self, *args):
        return
    def debug(self, *args):
        return
    def error(self, *args):
        return
    def setLevel(cls, level):
        cls.level = level
    setLevel = classmethod(setLevel)
_LoggerClass = ILogger


class BasicLogger(ILogger):
    def __init__(self, msg, out=sys.stdout):
        self.msg, self.out = msg, out

    def warning(self, msg, *args):
        if self.level < 1: return
        print >>self, self.WARN, self.msg,
        print >>self, msg %args
    WARN = 'WARN'
    def debug(self, msg, *args):
        if self.level < 2: return
        print >>self, self.DEBUG, self.msg,
        print >>self, msg %args
    DEBUG = 'DEBUG'
    def error(self, msg, *args):
        print >>self, self.ERROR, self.msg,
        print >>self, msg %args
    ERROR = 'ERROR'

    def write(self, *args):
        '''Write convenience function; writes strings.
        '''
        for s in args: self.out.write(s)


def setBasicLogger():
    '''Use Basic Logger. 
    '''
    setLoggerClass(BasicLogger)
    BasicLogger.setLevel(0)

def setBasicLoggerWARN():
    '''Use Basic Logger.
    '''
    setLoggerClass(BasicLogger)
    BasicLogger.setLevel(1)

def setBasicLoggerDEBUG():
    '''Use Basic Logger.
    '''
    setLoggerClass(BasicLogger)
    BasicLogger.setLevel(2)

def setLoggerClass(loggingClass):
    '''Set Logging Class.
    '''
    assert issubclass(loggingClass, ILogger), 'loggingClass must subclass ILogger'
    global _LoggerClass
    _LoggerClass = loggingClass

def setLevel(level=0):
    '''Set Global Logging Level.
    '''
    ILogger.level = level

def getLogger(msg):
    '''Return instance of Logging class.
    '''
    return _LoggerClass(msg)





#!/usr/bin/python
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Handy utility functions."""

__author__ = 'api.sgrinberg@gmail.com (Stan Grinberg)'

import os

from adspygoogle.adwords import LIB_HOME
from adspygoogle.common import Utils


def GetCurrencies():
  """Get a list of available currencies.

  Returns:
    list Available currencies.
  """
  return Utils.GetDataFromCsvFile(os.path.join(LIB_HOME, 'data',
                                               'currencies.csv'))


def GetErrorCodes():
  """Get a list of available error codes.

  Returns:
    list Available error codes.
  """
  return Utils.GetDataFromCsvFile(os.path.join(LIB_HOME, 'data',
                                               'error_codes.csv'))


def GetTimezones():
  """Get a list of available timezones.

  Returns:
    list Available timezones.
  """
  return Utils.GetDataFromCsvFile(os.path.join(LIB_HOME, 'data',
                                               'timezones.csv'))


def TransformJobOperationXsi(operation):
  """Ensures that the types in an JobOperation's inner Operations end correctly.

  This function ensures backwards compatibility with previous versions of the
  library which allowed you leave "Operation" off of an Operation's type. This
  is now considered deprecated style; always put a type's full name.

  Args:
    operation: dict The JobOperation object.
  """
  for item in operation['operand']['request']['operationStreams']:
    for sub_item in item['operations']:
      if ('xsi_type' in sub_item and
          not sub_item['xsi_type'].endswith('Operation')):
        sub_item['xsi_type'] = '%sOperation' % sub_item['xsi_type']
      elif 'type' in sub_item and not sub_item['type'].endswith('Operation'):
        sub_item['type'] = '%sOperation' % sub_item['type']


def TransformUserListRuleOperands(user_list):
  """Ensures that the key in the ruleOperands dicts is set correctly.

  This function ensures backwards compatibility with previous versions of the
  library which allowed you place a UserList or UserInterest directly into the
  ruleOperands field of a UserListLogicalRule instead of placing it inside of a
  LogicalUserListOperand object. This is now considered deprecated style; always
  put the LogicalUserListOperand object in the middle.

  Args:
    user_list: dict A UserList or UserInterest object.
  """
  if 'rules' in user_list:
    rules = []
    for item in user_list['rules']:
      if 'ruleOperands' in item:
        operands = []
        for sub_item in item['ruleOperands']:
          if 'xsi_type' in sub_item:
            operand = {}
            if sub_item['xsi_type'] in ('UserInterest',):
              TransformUserListRuleOperands(sub_item)
              operand['UserInterest'] = sub_item
            else:
              TransformUserListRuleOperands(sub_item)
              operand['UserList'] = sub_item
            operands.append(operand)
            item['ruleOperands'] = operands
      rules.append(item)
    user_list['rules'] = rules


def ExtractGroupNameFromUrl(url):
  """Takes an AdWords service's URL and extracts the group it belongs to.

  Args:
    url: string The URL of an AdWords service.

  Returns:
    string The group this AdWords service belongs to.
  """
  return url.split('/')[-2]



#!/usr/bin/python
#
# Copyright 2010 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Script to configure DFP API Python client library."""

__author__ = 'api.sgrinberg@gmail.com (Stan Grinberg)'

import os
import pickle


from adspygoogle.common import SanityCheck
from adspygoogle.common.Errors import InvalidInputError
from adspygoogle.common.Logger import Logger


HOME = os.path.expanduser('~')
AUTH_PKL = os.path.join(HOME, 'dfp_api_auth.pkl')
CONFIG_PKL = os.path.join(HOME, 'dfp_api_config.pkl')
LOG_HOME = os.path.join(HOME, 'logs')
LOG_NAME = 'dfp_api_lib'


logger = Logger(os.path.join(LOG_HOME))

# Load existing authentication credentials from dfp_api_auth.pkl.
old_auth = {}
if os.path.exists(AUTH_PKL):
  try:
    fh = open(AUTH_PKL, 'r')
    try:
      old_auth = pickle.load(fh)
    finally:
      fh.close()
  except IOError, e:
    logger.Log(LOG_NAME, e, log_level=Logger.ERROR)

# Prompt user for authentication and configuration values.
print """
--~--~---------~--~----~------------~-------~--~----~
All requests that are sent to the DFP API web services
must include SOAP header elements. Currently accepted
header elements are networkCode and applicationName.
Detailed descriptions of these properties can be found at:
https://developers.google.com/doubleclick-publishers/docs/soap_xml

All requests must use OAuth 2.0 for authentication. We
use clientId, clientSecret, and refreshToken to enable
this feature.

Starting in v201203, networkCodes are required for
all calls other than GetAllNetworks and
MakeTestNetwork. In earlier versions, networkCode is
optional if the user belongs to only one network.

For the applicationName header, the client library name
and its version is automatically prefixed. Supply
just the arbitrary string that identifies the
customer sending the request.

To overwrite an existing header element, explicitly
type new value (or 'none' to clear) at the prompt.
The default behavior is to keep old values.
-~----------~----~----~----~------~----~------~--~---\n"""
prompts = (('OAuth 2.0 client ID (retrieved from '
            'code.google.com/apis/console under the API Access tab)',
            'clientId', 'auth'),
           ('OAuth 2.0 client secret (retrieved from '
            'code.google.com/apis/console under the API Access tab)',
            'clientSecret', 'auth'),
           ('OAuth 2.0 refresh token (if you don\'t already have one, run '
            'scripts/common/generate_refresh_token.py', 'refreshToken', 'auth'),
           ('Network code', 'networkCode', 'auth'),
           ('Application name', 'applicationName', 'auth'),
           ('Select XML parser [1=PyXML, 2=ElementTree]', 'xml_parser',
            'config'),
           ('Enable debugging mode', 'debug', 'config'),
           ('Enable SOAP XML logging mode', 'xml_log', 'config'),
           ('Enable API request logging mode', 'request_log', 'config'))
auth = {}
config = {
    'home': HOME,
    'log_home': LOG_HOME
}
for prompt_msg, header, source in prompts:
  if source == 'auth':
    # Construct prompt message.
    try:
      prompt_msg = '%s [%s]: ' % (prompt_msg, old_auth[header])
    except (NameError, KeyError), e:
      prompt_msg = '%s: ' % prompt_msg

    # Prompt user to keep/update authentication credentials.
    auth[header] = raw_input(prompt_msg).rstrip('\r')

    if header in old_auth:
      if auth[header] == 'none':
        auth[header] = ''
      elif not auth[header]:
        auth[header] = old_auth[header]
    else:
      if auth[header] == 'none':
        auth[header] = ''

    if header == 'applicationName' and not auth[header].strip():
      msg = 'Application name must not be empty.'
      raise InvalidInputError(msg)
  elif source == 'config':
    # Prompt user to update configuration values.
    if header == 'xml_parser':
      res = raw_input('%s: ' % prompt_msg).rstrip('\r')
      if not SanityCheck.IsConfigUserInputValid(res, ['1', '2']):
        msg = 'Possible values are \'1\' or \'2\'.'
        raise InvalidInputError(msg)
    else:
      res = raw_input('%s [y/n]: ' % prompt_msg).rstrip('\r')
      if not SanityCheck.IsConfigUserInputValid(res, ['y', 'n']):
        msg = 'Possible values are \'y\' or \'n\'.'
        raise InvalidInputError(msg)
    config[header] = res

# Load new authentication credentials into dfp_api_auth.pkl.
try:
  fh = open(AUTH_PKL, 'w')
  try:
    pickle.dump(auth, fh)
  finally:
    fh.close()
except IOError, e:
  logger.Log(LOG_NAME, e, log_level=Logger.ERROR)

# Load new configuratation values into dfp_api_config.pkl.
try:
  fh = open(CONFIG_PKL, 'w')
  try:
    pickle.dump(config, fh)
  finally:
    fh.close()
except IOError, e:
  logger.Log(LOG_NAME, e, log_level=Logger.ERROR)
